@@ -24,27 +24,36 @@ functions = [
     lambda x: x.T,
     lambda x: da.transpose(x, (1, 2, 0)),
     lambda x: x.sum(),
+    lambda x: da.empty_like(x),
+    lambda x: da.ones_like(x),
+    lambda x: da.zeros_like(x),
+    lambda x: da.full_like(x, 5),
     pytest.param(
         lambda x: x.mean(),
-        marks=pytest.mark.xfail(
-            reason="requires NumPy>=1.17 and CuPy support for shape argument in *_like functions."
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+            reason="NEP-18 support is not available in NumPy or CuPy older than "
+            "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
         ),
     ),
     pytest.param(
         lambda x: x.moment(order=0),
-        marks=pytest.mark.xfail(reason="see https://github.com/dask/dask/issues/4875"),
     ),
     lambda x: x.moment(order=2),
     pytest.param(
         lambda x: x.std(),
-        marks=pytest.mark.xfail(
-            reason="requires NumPy>=1.17 and CuPy support for shape argument in *_like functions."
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+            reason="NEP-18 support is not available in NumPy or CuPy older than "
+            "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
         ),
     ),
     pytest.param(
         lambda x: x.var(),
-        marks=pytest.mark.xfail(
-            reason="requires NumPy>=1.17 and CuPy support for shape argument in *_like functions."
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+            reason="NEP-18 support is not available in NumPy or CuPy older than "
+            "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
         ),
     ),
     pytest.param(
@@ -66,7 +75,6 @@ functions = [
     lambda x: x.map_blocks(lambda x: x * 2),
     pytest.param(
         lambda x: x.round(1),
-        marks=pytest.mark.xfail(reason="cupy doesn't support round"),
     ),
     lambda x: x.reshape((x.shape[0] * x.shape[1], x.shape[2])),
     # Rechunking here is required, see https://github.com/dask/dask/issues/2561
@@ -78,9 +86,6 @@ functions = [
     lambda x: x.rechunk((2, 2, 1)),
     pytest.param(
         lambda x: da.einsum("ijk,ijk", x, x),
-        marks=pytest.mark.xfail(
-            reason="depends on resolution of https://github.com/numpy/numpy/issues/12974"
-        ),
     ),
     lambda x: np.isneginf(x),
     lambda x: np.isposinf(x),
@@ -132,6 +137,60 @@ functions = [
             not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
         ),
     ),
+    pytest.param(
+        lambda x: np.max(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.min(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.prod(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.any(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.all(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.nansum(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.nanprod(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.nanmin(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
+    pytest.param(
+        lambda x: np.nanmax(x),
+        marks=pytest.mark.skipif(
+            not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+        ),
+    ),
 ]
 
 
@@ -146,9 +205,13 @@ def test_basic(func):
     ddn = func(dn)
 
     assert type(ddc._meta) == cupy.core.core.ndarray
-    assert_eq(ddc, ddc)  # Check that _meta and computed arrays match types
 
-    assert_eq(ddc, ddn)
+    if ddc.dask.keys()[0][0].startswith("empty"):
+        # We can't verify for data correctness when testing empty_like
+        assert type(ddc._meta) == type(ddc.compute())
+    else:
+        assert_eq(ddc, ddc)  # Check that _meta and computed arrays match types
+        assert_eq(ddc, ddn)
 
 
 @pytest.mark.parametrize("dtype", ["f4", "f8"])
@@ -244,12 +307,10 @@ def test_diagonal():
     assert_eq(da.diagonal(v, 1, 2, 1), np.diagonal(v, 1, 2, 1))
 
 
-@pytest.mark.xfail(reason="no shape argument support *_like functions on CuPy yet")
 @pytest.mark.skipif(
-    np.__version__ < "1.17", reason="no shape argument for *_like functions"
-)
-@pytest.mark.skipif(
-    not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+    not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+    reason="NEP-18 support is not available in NumPy or CuPy older than "
+    "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
 )
 def test_tril_triu():
     A = cupy.random.randn(20, 20)
@@ -264,12 +325,10 @@ def test_tril_triu():
             assert_eq(da.tril(dA, k), np.tril(A, k))
 
 
-@pytest.mark.xfail(reason="no shape argument support *_like functions on CuPy yet")
 @pytest.mark.skipif(
-    np.__version__ < "1.17", reason="no shape argument for *_like functions"
-)
-@pytest.mark.skipif(
-    not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+    not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+    reason="NEP-18 support is not available in NumPy or CuPy older than "
+    "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
 )
 def test_tril_triu_non_square_arrays():
     A = cupy.random.randint(0, 11, (30, 35))
@@ -378,12 +437,10 @@ def test_nearest():
     assert_eq(e, expected)
 
 
-@pytest.mark.xfail(reason="no shape argument support *_like functions on CuPy yet")
 @pytest.mark.skipif(
-    np.__version__ < "1.17", reason="no shape argument for *_like functions"
-)
-@pytest.mark.skipif(
-    not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+    not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+    reason="NEP-18 support is not available in NumPy or CuPy older than "
+    "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
 )
 def test_constant():
     x = cupy.arange(64).reshape((8, 8))
@@ -397,12 +454,10 @@ def test_constant():
     assert_eq(e[-1, :], np.ones(8, dtype=x.dtype) * 10)
 
 
-@pytest.mark.xfail(reason="no shape argument support *_like functions on CuPy yet")
 @pytest.mark.skipif(
-    np.__version__ < "1.17", reason="no shape argument for *_like functions"
-)
-@pytest.mark.skipif(
-    not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+    not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+    reason="NEP-18 support is not available in NumPy or CuPy older than "
+    "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
 )
 def test_boundaries():
     x = cupy.arange(64).reshape((8, 8))
@@ -490,11 +545,10 @@ def test_random_shapes(shape):
     assert x.shape == shape
 
 
-@pytest.mark.xfail(
-    reason="CuPy division by zero on tensordot(), https://github.com/cupy/cupy/pull/2209"
-)
 @pytest.mark.skipif(
-    not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+    not IS_NEP18_ACTIVE or cupy.__version__ < "6.1.0",
+    reason="NEP-18 support is not available in NumPy or CuPy older than "
+    "6.1.0 (requires https://github.com/cupy/cupy/pull/2209)",
 )
 @pytest.mark.parametrize(
     "m,n,chunks,error_type",
@@ -858,12 +912,10 @@ def test_cupy_sparse_concatenate(axis):
     assert (z.toarray() == z_expected.toarray()).all()
 
 
-@pytest.mark.xfail(reason="no shape argument support *_like functions on CuPy yet")
 @pytest.mark.skipif(
-    np.__version__ < "1.17", reason="no shape argument for *_like functions"
-)
-@pytest.mark.skipif(
-    not IS_NEP18_ACTIVE, reason="NEP-18 support is not available in NumPy"
+    not IS_NEP18_ACTIVE or cupy.__version__ < "6.4.0",
+    reason="NEP-18 support is not available in NumPy or CuPy older than "
+    "6.4.0 (requires https://github.com/cupy/cupy/pull/2418)",
 )
 def test_bincount():
     x = cupy.array([2, 1, 5, 2, 1])
@@ -874,3 +926,31 @@ def test_bincount():
 
     assert da.bincount(d, minlength=6).name != da.bincount(d, minlength=7).name
     assert da.bincount(d, minlength=6).name == da.bincount(d, minlength=6).name
+
+
+@pytest.mark.parametrize("sp_format", ["csr", "csc"])
+def test_sparse_dot(sp_format):
+    pytest.importorskip("cupyx")
+
+    if sp_format == "csr":
+        sp_matrix = cupyx.scipy.sparse.csr_matrix
+    elif sp_format == "csc":
+        sp_matrix = cupyx.scipy.sparse.csc_matrix
+    dtype = "f"
+    density = 0.3
+    x_shape, x_chunks = (4, 8), (2, 4)
+    y_shape, y_chunks = (8, 6), (4, 3)
+    x = cupy.random.random(x_shape, dtype=dtype)
+    y = cupy.random.random(y_shape, dtype=dtype)
+    x[x < 1 - density] = 0
+    y[y < 1 - density] = 0
+    z = x.dot(y)
+
+    da_x = da.from_array(x, chunks=x_chunks, asarray=False, fancy=False)
+    da_y = da.from_array(y, chunks=y_chunks, asarray=False, fancy=False)
+    da_x = da_x.map_blocks(sp_matrix, dtype=dtype)
+    da_y = da_y.map_blocks(sp_matrix, dtype=dtype)
+    da_z = da.dot(da_x, da_y).compute()
+
+    assert cupyx.scipy.sparse.isspmatrix(da_z)
+    assert_eq(z, da_z.todense())
