@@ -449,35 +449,59 @@ def test_dask_svd_self_consistent(m, n):
         assert d_e.dtype == e.dtype
 
 
-@pytest.mark.slow
-def test_svd_compressed():
-    m, n = 2000, 250
-    r = 10
-    np.random.seed(4321)
-    mat1 = np.random.randn(m, r)
-    mat2 = np.random.randn(r, n)
-    mat = mat1.dot(mat2)
-    data = da.from_array(mat, chunks=(500, 50))
+@pytest.mark.parametrize("iterator", [("none", 0), ("power", 1), ("QR", 1)])
+def test_svd_compressed_compute(iterator):
+    x = da.ones((100, 100), chunks=(10, 10))
+    u, s, v = da.linalg.svd_compressed(
+        x, k=2, iterator=iterator[0], n_power_iter=iterator[1], compute=True, seed=123
+    )
+    uu, ss, vv = da.linalg.svd_compressed(
+        x, k=2, iterator=iterator[0], n_power_iter=iterator[1], seed=123
+    )
 
-    u, s, vt = svd_compressed(data, r, seed=4321, n_power_iter=2)
+    assert len(v.dask) < len(vv.dask)
+    assert_eq(v, vv)
 
-    usvt = da.dot(u, da.dot(da.diag(s), vt))
 
-    tol = 0.2
-    assert_eq(
-        da.linalg.norm(usvt), np.linalg.norm(mat), rtol=tol, atol=tol
-    )  # average accuracy check
+@pytest.mark.parametrize("iterator", [("power", 2), ("QR", 2)])
+def test_svd_compressed(iterator):
+    # Note: iterator="none" is not tested due to its
+    # difficulty with respect to the stochastic process of sampling,
+    # leading to false positives and false negatives in this test.
+    m, n = 100, 50
+    r = 5
+    a = da.random.random((m, n), chunks=(m, n))
 
-    u = u[:, :r]
-    s = s[:r]
-    vt = vt[:r, :]
+    # calculate approximation and true singular values
+    u, s, vt = svd_compressed(
+        a, 2 * r, iterator=iterator[0], n_power_iter=iterator[1], seed=4321
+    )  # worst case
+    s_true = scipy.linalg.svd(a.compute(), compute_uv=False)
 
-    s_exact = np.linalg.svd(mat)[1]
-    s_exact = s_exact[:r]
+    # compute the difference with original matrix
+    norm = scipy.linalg.norm((a - (u[:, :r] * s[:r]) @ vt[:r, :]).compute(), 2)
 
-    assert_eq(np.eye(r, r), da.dot(u.T, u))  # u must be orthonormal
-    assert_eq(np.eye(r, r), da.dot(vt, vt.T))  # v must be orthonormal
-    assert_eq(s, s_exact)  # s must contain the singular values
+    # ||a-a_hat||_2 <= (1+tol)s_{k+1}: based on eq. 1.10/1.11:
+    # Halko, Nathan, Per-Gunnar Martinsson, and Joel A. Tropp.
+    # "Finding structure with randomness: Probabilistic algorithms for constructing
+    # approximate matrix decompositions." SIAM review 53.2 (2011): 217-288.
+    frac = norm / s_true[r + 1] - 1
+    # Tolerance determined via simulation to be slightly above max norm of difference matrix in 10k samples.
+    # See https://github.com/dask/dask/pull/6799#issuecomment-726631175 for more details.
+    tol = 0.4
+    assert frac < tol
+
+    assert_eq(np.eye(r, r), da.dot(u[:, :r].T, u[:, :r]))  # u must be orthonormal
+    assert_eq(np.eye(r, r), da.dot(vt[:r, :], vt[:r, :].T))  # v must be orthonormal
+
+
+@pytest.mark.parametrize(
+    "input_dtype, output_dtype", [(np.float32, np.float32), (np.float64, np.float64)]
+)
+def test_svd_compressed_dtype_preservation(input_dtype, output_dtype):
+    x = da.random.random((50, 50), chunks=(50, 50)).astype(input_dtype)
+    u, s, vt = svd_compressed(x, 1, seed=4321)
+    assert u.dtype == s.dtype == vt.dtype == output_dtype
 
 
 @pytest.mark.parametrize("chunks", [(10, 50), (50, 10), (-1, -1)])
@@ -503,7 +527,7 @@ def test_svd_compressed_deterministic():
 @pytest.mark.parametrize("chunks", [(5, 10), (10, 5)])
 def test_svd_compressed_shapes(m, n, k, chunks):
     x = da.random.random(size=(m, n), chunks=chunks)
-    u, s, v = svd_compressed(x, k=k, n_power_iter=1, compute=True, seed=1)
+    u, s, v = svd_compressed(x, k, n_power_iter=1, compute=True, seed=1)
     u, s, v = da.compute(u, s, v)
     r = min(m, n, k)
     assert u.shape == (m, r)
@@ -511,13 +535,17 @@ def test_svd_compressed_shapes(m, n, k, chunks):
     assert v.shape == (r, n)
 
 
-def test_svd_compressed_compute():
+@pytest.mark.parametrize("iterator", [("none", 0), ("power", 1), ("QR", 1)])
+def test_svd_compressed_compute(iterator):
     x = da.ones((100, 100), chunks=(10, 10))
-    u, s, v = da.linalg.svd_compressed(x, k=2, n_power_iter=0, compute=True, seed=123)
-    uu, ss, vv = da.linalg.svd_compressed(x, k=2, n_power_iter=0, seed=123)
+    u, s, v = da.linalg.svd_compressed(
+        x, 2, iterator=iterator[0], n_power_iter=iterator[1], compute=True, seed=123
+    )
+    uu, ss, vv = da.linalg.svd_compressed(
+        x, 2, iterator=iterator[0], n_power_iter=iterator[1], seed=123
+    )
 
     assert len(v.dask) < len(vv.dask)
-
     assert_eq(v, vv)
 
 
@@ -818,6 +846,19 @@ def test_lstsq(nrow, ncol, chunk):
     dx, dr, drank, ds = da.linalg.lstsq(dA, db)
     assert drank.compute() == rank
 
+    # 2D case
+    A = np.random.randint(1, 20, (nrow, ncol))
+    b2D = np.random.randint(1, 20, (nrow, ncol // 2))
+    dA = da.from_array(A, (chunk, ncol))
+    db2D = da.from_array(b2D, (chunk, ncol // 2))
+    x, r, rank, s = np.linalg.lstsq(A, b2D, rcond=-1)
+    dx, dr, drank, ds = da.linalg.lstsq(dA, db2D)
+
+    assert_eq(dx, x)
+    assert_eq(dr, r)
+    assert drank.compute() == rank
+    assert_eq(ds, s)
+
 
 def test_no_chunks_svd():
     x = np.random.random((100, 10))
