@@ -6,7 +6,6 @@ from ..core import DataFrame, Series
 from ...base import tokenize
 from ...utils import derived_from
 from ...highlevelgraph import HighLevelGraph
-from .._compat import PANDAS_GT_0240
 from .. import methods
 
 
@@ -32,19 +31,14 @@ def _resample_series(
     out = getattr(series.resample(rule, **resample_kwargs), how)(
         *how_args, **how_kwargs
     )
-    if PANDAS_GT_0240:
-        new_index = pd.date_range(
-            start.tz_localize(None),
-            end.tz_localize(None),
-            freq=rule,
-            closed=reindex_closed,
-            name=out.index.name,
-        ).tz_localize(start.tz, nonexistent="shift_forward")
 
-    else:
-        new_index = pd.date_range(
-            start, end, freq=rule, closed=reindex_closed, name=out.index.name
-        )
+    new_index = pd.date_range(
+        start.tz_localize(None),
+        end.tz_localize(None),
+        freq=rule,
+        closed=reindex_closed,
+        name=out.index.name,
+    ).tz_localize(start.tz, nonexistent="shift_forward")
 
     if not out.index.isin(new_index).all():
         raise ValueError(
