@@ -270,7 +270,7 @@ def test_tokenize_pandas_invalid_unicode():
 @pytest.mark.skipif("not pd")
 def test_tokenize_pandas_mixed_unicode_bytes():
     df = pd.DataFrame(
-        {u"ö".encode("utf8"): [1, 2, 3], u"ö": [u"ö", u"ö".encode("utf8"), None]},
+        {"ö".encode("utf8"): [1, 2, 3], "ö": ["ö", "ö".encode("utf8"), None]},
         index=[1, 2, 3],
     )
     tokenize(df)
@@ -286,13 +286,8 @@ def test_tokenize_pandas_no_pickle():
     tokenize(df)
 
 
-@pytest.mark.skipif("not pd")
+@pytest.mark.skipif("not dd")
 def test_tokenize_pandas_extension_array():
-    from dask.dataframe._compat import PANDAS_GT_100, PANDAS_GT_0240
-
-    if not PANDAS_GT_0240:
-        pytest.skip("requires pandas>=1.0.0")
-
     arrays = [
         pd.array([1, 0, None], dtype="Int64"),
         pd.array(["2000"], dtype="Period[D]"),
@@ -305,7 +300,7 @@ def test_tokenize_pandas_extension_array():
         ),
     ]
 
-    if PANDAS_GT_100:
+    if dd._compat.PANDAS_GT_100:
         arrays.extend(
             [
                 pd.array(["a", "b", None], dtype="string"),
@@ -403,7 +398,7 @@ def test_tokenize_range():
 
 @pytest.mark.skipif("not np")
 def test_tokenize_object_array_with_nans():
-    a = np.array([u"foo", u"Jos\xe9", np.nan], dtype="O")
+    a = np.array(["foo", "Jos\xe9", np.nan], dtype="O")
     assert tokenize(a) == tokenize(a)
 
 
@@ -677,8 +672,8 @@ def test_compute_dataframe():
     ddf1 = ddf.a + 1
     ddf2 = ddf.a + ddf.b
     out1, out2 = compute(ddf1, ddf2)
-    pd.testing.assert_series_equal(out1, df.a + 1)
-    pd.testing.assert_series_equal(out2, df.a + df.b)
+    dd._compat.tm.assert_series_equal(out1, df.a + 1)
+    dd._compat.tm.assert_series_equal(out2, df.a + df.b)
 
 
 @pytest.mark.skipif("not dd or not da")
@@ -694,7 +689,7 @@ def test_compute_array_dataframe():
 
 @pytest.mark.skipif("not dd")
 def test_compute_dataframe_valid_unicode_in_bytes():
-    df = pd.DataFrame(data=np.random.random((3, 1)), columns=[u"ö".encode("utf8")])
+    df = pd.DataFrame(data=np.random.random((3, 1)), columns=["ö".encode("utf8")])
     dd.from_pandas(df, npartitions=4)
 
 
